type 'a tree =
	(* étiquette, liste des enfants *)
	| Node of 'a * 'a tree list 

type 'a file =
	'a list * 'a list


let file_vide () = ([], [])

(* enfile x dans f *)
let enfiler (x: 'a) (f: 'a file) =
	let t, q = f in
	(t, x::q)

(* défile la tete de f et la renvoie *)
let rec defiler (f: 'a file) : 'a * 'a file =
	match f with
	| [], [] -> failwith "file vide"
	| x::t, q -> (x, (t, q))
	| [], q -> let t = List.rev q in defiler (t, [])

let rec defiler ((t, q): 'a file) : 'a * 'a file =
	match t with
	| [] -> 
		begin match List.rev q with
		| [] -> failwith "File vide"
		| x :: r -> x, (r, [])	
		end
	| x :: r -> (x, (r, q))



(* true si f est vide, false sinon *)
let est_vide (f: 'a file) = 
	f = ([], [])


(* liste des étiquettes de a par ordre de 
   profondeurs croissantes *)
let parcours_largeur (a: 'a tree) =
	(* liste des étiquettes des éléments de f.
	   Simule la boucle de l'algorithme itératif. *)
	let rec parcours_file (f: 'a tree file) =
		if est_vide f then [] else 
		let (Node(x, l), f) = defiler f in
		(* Enfiler tous les éléments de l dans la file *)
		let prochaine_file = List.fold_right enfiler (List.rev l) f in
		(* ajouter x au résultat, et continuer le parcours *)
		x :: parcours_file prochaine_file
	in
	parcours_file (enfiler a (file_vide ()))

let test () =
	let t = 
	Node(1, [
		Node(2, [
			Node(5, []);
			Node(6, []);
		]);
		Node(3, [
			Node(7, []);
			Node(8, []);
		]);
		Node(4, [
			Node(9, []);
			Node(10, []);
		]);
	])
	in
	assert (parcours_largeur t = [1; 2; 3; 4; 5; 6; 7; 8; 9; 10])